﻿using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using System.Linq;
using System.Text.RegularExpressions;
namespace TechAmit.SharePoint.LogicalFieldUpdate.BulkRun
{
    public partial class BulkRunUserControl : UserControl
    {
        MsgControl msg;
        static class UpdateType
        {
            public const string CurrentUser = "CurrentUser";
            public const string System = "System";
            public const string OverwriteVersion = "OverwriteVersion";
        }

        #region Events
        protected void Page_Load(object sender, EventArgs e)
        {
            // Create new instatnce of message control
            msg = new MsgControl();

            if (!Page.IsPostBack)
            {
                try
                {
                    PreSet();
                }
                catch(Exception ex) { AddUnexpectedError(ex); }
            }
        }

        protected void Page_PreRender(object sender, EventArgs e)
        {
            phMsg.Controls.Add(msg.MsgList);
        }

        protected void btnRun_Click(object sender, EventArgs e)
        {

            if (ValidateInput() == false) { return; }

            var logicalList = SPContext.Current.Web.Lists.TryGetList(Constants.LogicListName);
            var lr = new LogicFieldUpdater(logicalList);

            #region Get Items Using View
            // Get items from the list using view defined
            //var list = SPContext.Current.Web.Lists.TryGetList(listName);
            //var view = list.Views[ddlTargetListView.SelectedValue];
            var items = GetItemsToUpdate();
            //var items = TargetList.GetItems();
            #endregion

            #region Update Items
            int totItemsUpdated = 0;
            int totItemsNotUpdated = 0;
            foreach (SPListItem item in items)
            {

                try
                {
                    if (lr.RunLogicalUpdate(item))
                    {
                        // Logical update has been made,
                        // so need to update item.
                        UpdateItem(item);
                        totItemsUpdated++;
                    }
                }
                catch
                {
                    totItemsNotUpdated++;
                }

            }
            #endregion

            #region Show messages for updated and not updated

            // Show message that no item update
            if (totItemsUpdated == 0)
            {
                msg.AddAlert(MsgReader.GetById(9));
            }

            // Show message for no of items updated
            if (totItemsUpdated > 0)
            {
                msg.AddSuccess(MsgReader.GetById(4, totItemsUpdated));
            }

            // Show message for no of items not updated or 
            // error occured while updating
            if (totItemsNotUpdated > 0)
            {
                msg.AddError(MsgReader.GetById(8, totItemsNotUpdated));
            }
            #endregion
        }


        protected void ddlTargetList_SelectedIndexChanged(object sender, EventArgs e)
        {
            // If list is not selected
            if (string.IsNullOrEmpty(SelectedListName)) return;

            // Make sure selected lists exist
            if (TargetList == null)
            {
                // The list user selected list is no more exists or invalid
                // Fill the list again
                msg.AddError(MsgReader.GetById(5, SelectedListName));
                FillList();
                return;
            }

            // Fill views
            FillView();

            // File success flag fields
            FillLogFields();

        }
        #endregion


        #region Private Properties
        
        // A message reader
        MsgReader _msgReader;
        private MsgReader MsgReader
        {
            get
            {
                if (_msgReader == null)
                {
                    _msgReader = new MsgReader(new MsgReader.DelgRead(GetMsg));
                }
                return _msgReader;
            }
        }

        // Selected list 
        string _selectedListName;
        private string SelectedListName
        {
            get
            {
                // If already set, return that value
                if (!string.IsNullOrEmpty(_selectedListName)) return _selectedListName;

                if (ddlTargetList.Items.Count == 0)
                {
                    _selectedListName = string.Empty;
                }
                else
                {
                    _selectedListName = ddlTargetList.SelectedValue;
                }

                return _selectedListName;
                
            }
        }

        // The logic list
        SPList _logicList;
        private SPList LogicList
        {
            get
            {
                // If already set, return that
                if (_logicList != null) return _logicList;

                // Get the list
                _logicList = SPContext.Current.Web.Lists.TryGetList(Constants.LogicListName);

                // return list
                return _logicList;
            }
        }

        // The target list on which bulk run to be performed
        SPList _targetList;
        private SPList TargetList
        {
            get
            {
                // If already set, return that value
                if (_targetList != null) return _targetList;

                // Make sure valid list is alread selected
                if (string.IsNullOrEmpty(SelectedListName)) return null;

                // Set the list according to selected list
                _targetList = SPContext.Current.Web.Lists.TryGetList(SelectedListName);

                return _targetList;
            }
        }

        // Target list name
        string _targetListName;
        private string TargetListName
        {
            get
            {
                if (_targetListName == null)
                {
                    _targetListName = ddlTargetList.SelectedValue;
                }

                return _targetListName;
            }
        }
        
        // View name
        string _viewName;
        private string ViewName
        {
            get
            {
                if (_viewName == null)
                {
                    _viewName = ddlTargetListView.SelectedValue;
                }

                return _viewName;
            }
        }

        // Success mark field name
        string _successMarkFieldName;
        private string SuccessMarkFieldName
        {
            get
            {
                if (_successMarkFieldName == null)
                {
                    _successMarkFieldName = ddlSuccessFlag.SelectedValue;
                }

                return _successMarkFieldName;
            }
        }

        // Success mark field name
        SPField _successMarkField;
        private SPField SuccessMarkField
        {
            get
            {
                if (_successMarkField == null)
                {
                    if (!string.IsNullOrEmpty(SuccessMarkFieldName))
                    {
                        _successMarkField = TargetList.Fields[SuccessMarkFieldName];
                    }
                }

                return _successMarkField;
            }
        }

        // Update type
        string _updateType;
        private string LogicalUpdateType
        {
            get
            {
                if (_updateType == null)
                {
                    _updateType = ddlUpdateType.SelectedValue;
                }

                return _updateType;
            }
        }
        #endregion

        #region Private Methods
        // This methoed is called after logical udpation
        // is made. Here we just call the Update methoed of
        // SPListItem
        private void UpdateItem(SPListItem item)
        {
            // flag if we need to mark success of logical update
            // If field is defined, we need to put the result
            bool IsMarkSuccessRequired = (SuccessMarkField != null);

            using (DisabledEventScope scope = new DisabledEventScope())
            {
                // Check if need to mark success
                if (IsMarkSuccessRequired)
                {
                    item[SuccessMarkFieldName] = true;
                }

                // Check if we need to log exception message
                #region Update using update type selected
                if (LogicalUpdateType == UpdateType.CurrentUser)
                {
                    item.Update();
                }
                else if (LogicalUpdateType == UpdateType.System)
                {
                    item.SystemUpdate(false);
                }
                else if (LogicalUpdateType == UpdateType.OverwriteVersion)
                {
                    item.UpdateOverwriteVersion();
                }
                #endregion
            }
        }

        private SPListItemCollection GetItemsToUpdate()
        {
            // Get view
            var view = TargetList.Views[ddlTargetListView.SelectedValue];

            #region Get "Where" filter expression from list view
            // Get the filter expression from view
            string filterCaml = string.Empty;

            // Regex pattern to extract the filter
            string pattern = @"<Where>\s*<.*</Where>";

            // Regex options: Ignore Case & Singleline
            var rxOptions = RegexOptions.IgnoreCase | RegexOptions.Singleline;
            var rx = new Regex(pattern,rxOptions);

            // find
            var match = rx.Match(view.Query);

            if (match != null && match.Success)
            {
                filterCaml = match.Value;
            }
            #endregion

            // Check if filter express found
            SPListItemCollection items;
            if (filterCaml == string.Empty)
            {
                // No filter express is defined in view
                // so get all the items
                items = TargetList.Items;
            }
            else
            {
                // Filter express is defined.
                // Use this filter express to get the 
                // the items
                var query = new SPQuery();
                query.Query = filterCaml;
                items = TargetList.GetItems(query);
            }

            // return items that need to be updated
            return items;

        }

        private bool ValidateInput()
        {
            bool IsValid = true;

            #region Check Required Values
            if (TargetListName == string.Empty)
            {
                msg.AddError(GetMsg(1));
                IsValid = false;
            }

            if (ViewName == string.Empty)
            { 
                msg.AddError(GetMsg(2));
                IsValid = false;
            }
            if (LogicalUpdateType == string.Empty) 
            { 
                msg.AddError(GetMsg(3));
                IsValid = false;
            }
            #endregion

            // If false, it means not all required values
            // are provided
            if (IsValid == false) { return false; }

            // Check if field that needs to me marked
            // if success is exists
            if (!string.IsNullOrEmpty(SuccessMarkFieldName))
            {
                if (SuccessMarkField == null)
                {
                    msg.AddError(MsgReader.GetById(6, SuccessMarkFieldName));
                    IsValid = false;
                }
            }

            return IsValid;
        }

        private void PreSet()
        {
            var web = SPContext.Current.Web;
            var list = web.Lists.TryGetList(Constants.LogicListName);

            hlLogicalList.Text = Constants.LogicListName;
            
            if (list == null)
            {
                // Show message list not exist
                msg.AddError(MsgReader.GetById(10, Constants.LogicListName));
                hlLogicalList.NavigateUrl = "javascript:alert('List does not exist');return false;";

                // Hide input controls
                pnlBulkRun.Visible = false;

                // Dont' proceed
                return;
            }
            else
            {
                hlLogicalList.NavigateUrl = list.DefaultViewUrl;
            }

            FillList();
        }

        private void AddUnexpectedError(Exception ex)
        {
            try
            {
                msg.AddError("An unexpceted error occured, please check next message for details.");
                msg.AddError(ex.Message);
            }
            catch { }
        }
        private void FillList()
        {
            // Remove all items
            ddlTargetList.Items.Clear();

            // Remove view items also
            ddlTargetListView.Items.Clear();

            // Make empty items as selected
            ddlTargetList.Items.Add(string.Empty);

            // Get the title and internal name of the fields that holds
            // the list title the logic applies to (in logic list)
            string fldName = Constants.LogicListFieldName.ListName;
            string fldInternalName = LogicList.Fields[fldName].InternalName;

            // Get all the lists name that are listed in logical list.
            // We need to show the list only linsted in our logical list.
            string[] fields = {fldInternalName};
            var items = LogicList.GetItems(fields);
            
            // If no items exist, don't proceed
            if (items == null || items.Count == 0) return;

            // Get the list name
            var distinct = from SPListItem li in items
                           select li[fldName].ToString();

            // Get unique list name
            foreach (string listName in distinct.Distinct().OrderBy(s=>s))
            {
                ddlTargetList.Items.Add(listName);
            }

        }

        private void FillView()
        {
            // Clear items
            ddlTargetListView.Items.Clear();

            // Add empty item
            ddlTargetListView.Items.Add(string.Empty);

            // Make sure list exists
            if (TargetList == null) return;

            var views = from SPView v in TargetList.Views
                        orderby v.ToString()
                        select v.ToString();

            foreach (string viewName in views.OrderBy(s=>s))
            {
                ddlTargetListView.Items.Add(viewName);
            }

        }

        private void FillLogFields()
        {
            // Clear previous items
            ddlSuccessFlag.Items.Clear();

            // Add empty item
            ddlSuccessFlag.Items.Add(string.Empty);

            // Make sure we have target list
            if (TargetList == null) return;

            // Add each Yes/No fields to list
            foreach (SPField fld in TargetList.Fields)
            {
                // If filed is hidden, ignore it
                if (fld.Hidden == true) { continue; }

                if (fld.Type == SPFieldType.Boolean)
                {
                    // Make sure the field is not system used field
                    if (!Constants.SystemBooleanFields.Contains(fld.Title))
                    {
                        ddlSuccessFlag.Items.Add(fld.Title);
                    }
                }
            }

        }

        private string GetMsg(int msgNo)
        {
            switch (msgNo)
            {
                case 1:
                    return "Please select a list";

                case 2:
                    return "Please select a view";

                case 3:
                    return "Please select update type";

                case 4:
                    return "Updation completed: Total {0} item(s) updated.";
                
                case 5:
                    return "Selected list '{0}' could not be found";

                case 6:
                    return "The field for success mark '{0}' not exists";

                case 7:
                    return "The field for exception message log '{0}' not exists";

                case 8:
                    return "Total {0} item(s) not updated or error occured while updating";

                case 9:
                    return "No item updated";

                case 10:
                    return "The logic list '{0}' does not exist. It gets automatically created when you activate the feature.";

                default:
                    return string.Empty;
            }
        }

        #endregion

    }
}
