using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
namespace TechAmit.SharePoint.LogicalFieldUpdate.Features.Feature1
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("06996d6f-cda0-4911-9bf1-e19147693a75")]
    public class Feature1EventReceiver : SPFeatureReceiver
    {
        // Uncomment the method below to handle the event raised after a feature has been activated.

        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            // Get web
            var web = properties.Feature.Parent as SPWeb;

            // Check if list already exist
            var list = web.Lists.TryGetList(Constants.LogicListName);
            
            // If list alredy exist, don't proceed
            if (list != null) return;

            // List does not exist, create it
            string listDescription = string.Empty;
            Guid listId = web.Lists.Add(Constants.LogicListName, listDescription , SPListTemplateType.GenericList);

            // Get the newly created list
            list = web.Lists[listId];

            var handler = new UserLogicListHandler(list);

            if (handler.CreateRequiredFields() == false)
            {
                list.Delete();
            }

        }


        // Uncomment the method below to handle the event raised before a feature is deactivated.

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            /*
            // Get web
            var web = properties.Feature.Parent as SPWeb;

            // Check if list already exist
            var list = web.Lists.TryGetList(Constants.LogicListName);

            if (list != null) list.Delete();
            */
        }


        // Uncomment the method below to handle the event raised after a feature has been installed.

        //public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised before a feature is uninstalled.

        //public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        //{
        //}

        // Uncomment the method below to handle the event raised when a feature is upgrading.

        //public override void FeatureUpgrading(SPFeatureReceiverProperties properties, string upgradeActionName, System.Collections.Generic.IDictionary<string, string> parameters)
        //{
        //}
    }
}
