﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using System.Linq;
namespace TechAmit.SharePoint.LogicalFieldUpdate
{
    class MsgControl
    {
        #region Private Variables
        BulletedList _bl;
        public enum MsgType
        {
            Info,
            Success,
            Error,
            Alert,
        }
        #endregion

        #region Public

        // Class initialization
        public MsgControl()
        {

            // Set to private our private variable
            _bl = new BulletedList();

            // Disable viewstate
            _bl.EnableViewState = false;

            // Put some styling to appear as list
            _bl.Attributes.Add("style", "margin:0;padding:0;list-style-type:none;");
        }
        public void AddMsg(ref string msg, MsgType msgType)
        {
            // Create new list item
            var li = new ListItem();

            // Add message
            li.Text = msg;

            // Add css styling
            li.Attributes.Add("style", GetStyle(msgType));

            // Finally add to our list
            MsgList.Items.Add(li);
        }
        public void AddAlert(ref string msg)
        {
            // Add an alert message
            AddMsg(ref msg, MsgType.Alert);
        }
        public void AddAlert(string msg)
        {
            // Add an alert message
            AddMsg(ref msg, MsgType.Alert);
        }
        public void AddInfo(ref string msg)
        {
            // Add an information message
            AddMsg(ref msg, MsgType.Info);
        }
        public void AddInfo(string msg)
        {
            // Add an information message
            AddMsg(ref msg, MsgType.Info);
        }
        public void AddError(ref string msg)
        {
            // Add an error message
            AddMsg(ref msg, MsgType.Error);
        }
        public void AddError(string msg)
        {
            // Add an error message
            AddMsg(ref msg, MsgType.Error);
        }
        public void AddSuccess(string msg)
        {
            // Add success message
            AddMsg(ref msg, MsgType.Success);
        }
        public void AddSuccess(ref string msg)
        {
            // Add a success message
            AddMsg(ref msg, MsgType.Success);
        }
        public void AddMsgFromControl(MsgControl ctl)
        {
            MsgList.Items.AddRange(ctl.MsgList.Items.OfType<ListItem>().ToArray());
        }
        #endregion

        #region Message

        #endregion

        #region Private
        public BulletedList MsgList
        {
            get { return _bl; }
        }

        // Returns css styling of message
        string GetStyle(MsgType msgType)
        {
            const string style = "padding:3px; margin-bottom:3px;color:#333333;" +
                            "border:1px solid {0};background-color:{1};";
            switch (msgType)
            {
                case MsgType.Info:
                    return string.Format(style, "#336699", "#E3F5F5");

                case MsgType.Alert:
                    return string.Format(style, "#FF9900", "#FFFBC9");

                case MsgType.Error:
                    return string.Format(style, "#E47297", "#FFDDDD");

                case MsgType.Success:
                    return string.Format(style, "#669966", "#D8F6D8");

                default:
                    return string.Empty;
            }
        }
        #endregion

    }
}
