﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
namespace TechAmit.SharePoint.LogicalFieldUpdate
{
    class UserLogicListHandler
    {
        public UserLogicListHandler(SPList list)
        {
            LogicList = list;
        }

        private SPList LogicList { get; set; }

        public bool CreateRequiredFields()
        {

            #region Add fields one by one
            // Add field one by one and exit (return) false,
            // if failed

            // Add field: List
            if (AddFieldList() == false) return false;

            // Add field: Condition
            if (AddFieldCondition() == false) return false;

            // Add field: Value type
            if (AddFieldValueType() == false) return false;

            // Add field: Field To Compare
            if (AddFieldFieldToCompare() == false) return false;

            // Add field: Value To Compare With
            if (AddFieldValueToCompareWith() == false) return false;

            // Add field: Update Field
            if (AddFieldUpdateField() == false) return false;

            // Add field: Update With
            if (AddFieldUpdateWith() == false) return false;

            // Add field: Periority
            if (AddFieldOrder() == false) return false;

            // Add field: Check Next
            if (AddFieldCheckNext() == false) return false;

            // Add field: Active
            if (AddFieldActive() == false) return false;

            // Add field: Condition Parameters
            if (AddFieldConditionParameters() == false) return false;

            // Add field: Check Event
            if (AddFieldCheckEvent() == false) return false;

            // Add field: Validate
            if (AddFieldValidate() == false) return false;
            #endregion

            try
            {
                LogicList.Update();
            }
            catch { return false; }

            #region Add fields to default view
            try
            {
                var view = LogicList.DefaultView;
                
                // Condition
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.Condition]);

                // Value type
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.ValueType]);

                // Field To Compare
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.FieldToCompare]);

                // Value To Compare With
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.ValueToComapreWith]);

                // Update Field
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.UpdateField]);

                // Update With
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.UpdateWith]);

                // Periority
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.Order]);

                // Check Next
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.CheckNext]);

                // Active
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.Active]);

                // Condition Parameters
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.ConditionParameter]);

                // When to check
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.CheckEvent]);

                // Validate
                view.ViewFields.Add(LogicList.Fields[Constants.LogicListFieldName.Validate]);

                view.Update();

                return true;
            }
            catch { return false; }
            #endregion
        }

        public bool AddFieldList()
        {
            // For "List" field we just need to 
            // rename existing Title field.

            try
            {
                // Try to get "Title" field
                var fldList = LogicList.Fields["Title"];

                // Make sure it exist, if not don't proceed
                if (fldList == null) return false;

                // "Title" field exists, rename it to "List"
                fldList.Title = Constants.LogicListFieldName.ListName;
                fldList.Description = "The title of list this logical test belongs to";
                fldList.Update();
                return true;
            }
            catch { return false; }
        }

        public bool AddFieldCondition()
        {

            try
            {
                string newFieldName = Constants.LogicListFieldName.Condition;
                LogicList.Fields.Add(newFieldName, SPFieldType.Choice, true);
                var fldCondition = LogicList.Fields[newFieldName] as SPFieldChoice;
                fldCondition.Description = "Type of match to perform";

                fldCondition.Choices.Add(Constants.ConditionTitle.Equal);
                fldCondition.Choices.Add(Constants.ConditionTitle.NotEqual);
                fldCondition.Choices.Add(Constants.ConditionTitle.Greater);
                fldCondition.Choices.Add(Constants.ConditionTitle.GreaterOrEqual);
                fldCondition.Choices.Add(Constants.ConditionTitle.Lesser);
                fldCondition.Choices.Add(Constants.ConditionTitle.LesserOrEqual);
                fldCondition.Choices.Add(Constants.ConditionTitle.StartsWith);
                fldCondition.Choices.Add(Constants.ConditionTitle.NotStartsWith);
                fldCondition.Choices.Add(Constants.ConditionTitle.EndsWith);
                fldCondition.Choices.Add(Constants.ConditionTitle.NotEndsWith);
                fldCondition.Choices.Add(Constants.ConditionTitle.Contains);
                fldCondition.Choices.Add(Constants.ConditionTitle.NotContains);
                fldCondition.Choices.Add(Constants.ConditionTitle.RegexMatches);
                fldCondition.Choices.Add(Constants.ConditionTitle.RegexNotMatches);
                fldCondition.Choices.Add(Constants.ConditionTitle.Null);
                fldCondition.Choices.Add(Constants.ConditionTitle.NotNull);
                fldCondition.Choices.Add(Constants.ConditionTitle.NullOrEmpty);
                fldCondition.Choices.Add(Constants.ConditionTitle.AnythingButNotNull);
                fldCondition.Choices.Add(Constants.ConditionTitle.AnythingIncludingNull);

                fldCondition.DefaultValue = Constants.ConditionTitle.Equal;

                // Required
                fldCondition.Required = true;

                // Update field
                fldCondition.Update();

                // Field successfully added
                return true;
            }

            catch { return false; } // Error occured
        }

        public bool AddFieldValueType()
        {
            try
            {
                string newFieldName = Constants.LogicListFieldName.ValueType;
                LogicList.Fields.Add(newFieldName, SPFieldType.Choice, true);
                var newField = LogicList.Fields[newFieldName] as SPFieldChoice;

                newField.Choices.Add(Constants.ConditionValueTypeTitle.AutoDetect);
                newField.Choices.Add(Constants.ConditionValueTypeTitle.String);
                newField.Choices.Add(Constants.ConditionValueTypeTitle.Numeric);
                newField.Choices.Add(Constants.ConditionValueTypeTitle.DateTime);

                newField.DefaultValue = Constants.ConditionValueTypeTitle.AutoDetect;

                // Required
                newField.Required = true;

                // Update field
                newField.Update();

                return true;
            }
            catch {return false;}
        }

        public bool AddFieldFieldToCompare()
        {
            try
            {
                string newFieldName = Constants.LogicListFieldName.FieldToCompare;
                LogicList.Fields.Add(newFieldName, SPFieldType.Text, true);
                var newField = LogicList.Fields[newFieldName];
                newField.Description = "The field to be used as source of match. (This field belongs to list defined above).";

                newField.Update();
                return true;
            }
            catch { return false; }
        }

        public bool AddFieldValueToCompareWith()
        {
            try
            {
                string newFieldName = Constants.LogicListFieldName.ValueToComapreWith;
                LogicList.Fields.Add(newFieldName, SPFieldType.Note, false);
                var newField = LogicList.Fields[newFieldName] as SPFieldMultiLineText;
                newField.Description = "Value to be matched with. " +
                                        "You can include fields using square bracket:[My Field]. " +
                                        "To escape, use [[My Field]].";
                newField.NumberOfLines = 4;
                newField.Update();
                return true;
            }
            catch { return false; }
        }

        public bool AddFieldUpdateWith()
        {
            try
            {
                string newFieldName = Constants.LogicListFieldName.UpdateWith;
                LogicList.Fields.Add(newFieldName, SPFieldType.Note, false);
                var fld = LogicList.Fields[newFieldName] as SPFieldMultiLineText;
                fld.Description = "The value by which the field should be updated. " +
                                    "You can include fields using square bracket:[My Field]. " +
                                    "To escape, use [[My Field]].";
                fld.NumberOfLines = 6;
                fld.Update();
                return true;
            }

            catch { return false; }
        }

        public bool AddFieldUpdateField()
        {
            try
            {
                string newFieldName = Constants.LogicListFieldName.UpdateField;
                LogicList.Fields.Add(newFieldName, SPFieldType.Text, true);

                var newField = LogicList.Fields[newFieldName];
                newField.Description = "The field which needs to be updated.";
                newField.Update();

                return true;
            }
            catch { return false; }
        }

        public bool AddFieldOrder()
        {
            try
            {
                string newFieldName = Constants.LogicListFieldName.Order;
                LogicList.Fields.Add(newFieldName, SPFieldType.Number, true);
                var newField = LogicList.Fields[newFieldName];
                newField.Description = "The order of execution. " + 
                    "You can use minus values. Order having -1 value will be executed before 0 or 1";

                newField.Update();
                return true;
            }
            catch { return false; }
        }

        public bool AddFieldCheckNext()
        {
            try
            {
                string newFieldName = Constants.LogicListFieldName.CheckNext;
                LogicList.Fields.Add(newFieldName, SPFieldType.Choice, true);
                var fld = LogicList.Fields[newFieldName] as SPFieldChoice;

                // Add choice items
                fld.Choices.Add(Constants.CheckNext.Always);
                fld.Choices.Add(Constants.CheckNext.IfTrue);
                fld.Choices.Add(Constants.CheckNext.IfFalse);

                // Set default
                fld.DefaultValue = Constants.CheckNext.Always;

                // Field description
                fld.Description = "Check next condition:" + Environment.NewLine +
                                    "Always: Always" + Environment.NewLine +
                                    "If True:Only if this condition is true" + Environment.NewLine +
                                    "If False:Only if this condition if false" + Environment.NewLine;
                fld.Update();
                return true;
            }
            catch { return false; }
        }

        public bool AddFieldActive()
        {
            try
            {
                string newFieldName = Constants.LogicListFieldName.Active;
                LogicList.Fields.Add(newFieldName, SPFieldType.Boolean, true);

                var fld = LogicList.Fields[newFieldName] as SPFieldBoolean;
                fld.DefaultValue = "1";
                fld.Update();

                return true;
            }
            catch { return false; }
        }

        public bool AddFieldConditionParameters()
        {
            // Add condition parameters field

            try
            {
                string newFieldName = Constants.LogicListFieldName.ConditionParameter;
                LogicList.Fields.Add(newFieldName, SPFieldType.MultiChoice, false);
                var fldConditionParameters = LogicList.Fields[newFieldName] as SPFieldMultiChoice;
                fldConditionParameters.Description = "Additional parameters for condition check";
                fldConditionParameters.Choices.Add(Constants.ConditionParametersTitle.TextCaseSensitive);
                fldConditionParameters.Choices.Add(Constants.ConditionParametersTitle.RegexSingleLine);
                fldConditionParameters.Choices.Add(Constants.ConditionParametersTitle.RegexIgnoreCase);
                fldConditionParameters.Choices.Add(Constants.ConditionParametersTitle.RegexCultureInvariant);
                fldConditionParameters.Choices.Add(Constants.ConditionParametersTitle.RegexEcmaScript);
                fldConditionParameters.Choices.Add(Constants.ConditionParametersTitle.RegexExplicitCapture);
                fldConditionParameters.Choices.Add(Constants.ConditionParametersTitle.RegexIgnorePatternWhitepsace);
                fldConditionParameters.Choices.Add(Constants.ConditionParametersTitle.RegexMultiLine);
                fldConditionParameters.Choices.Add(Constants.ConditionParametersTitle.RegexRightToLeft);

                // Default selected
                var defaultChoices = new SPFieldMultiChoiceValue();
                defaultChoices.Add(Constants.ConditionParametersTitle.TextCaseSensitive);
                fldConditionParameters.DefaultValue = defaultChoices.ToString();

                // Update field
                fldConditionParameters.Update();

                // Field successfully added
                return true;
            }
            catch { return false; } // Error occured
        }

        public bool AddFieldCheckEvent()
        {
            // Add when to check / run logical update field
            try
            {
                string newFieldName = Constants.LogicListFieldName.CheckEvent;
                LogicList.Fields.Add(newFieldName, SPFieldType.MultiChoice, false);
                var fldCheckWhen = LogicList.Fields[newFieldName] as SPFieldMultiChoice;

                // Add description
                fldCheckWhen.Description = "When to run logical update? " +
                    "If you are using Document Library please use after events, " +
                    "for reason please refer to: http://www.synergyonline.com/Blog/Lists/Posts/Post.aspx?ID=122 ";

                // Add options
                fldCheckWhen.Choices.Add(Constants.LogicalCheckEventsTitle.ItemAdding);
                fldCheckWhen.Choices.Add(Constants.LogicalCheckEventsTitle.ItemAdded);
                fldCheckWhen.Choices.Add(Constants.LogicalCheckEventsTitle.ItemUpdating);
                fldCheckWhen.Choices.Add(Constants.LogicalCheckEventsTitle.ItemUpdated);

                // Default selected
                var defaultChoices = new SPFieldMultiChoiceValue();
                defaultChoices.Add(Constants.LogicalCheckEventsTitle.ItemAdding);
                defaultChoices.Add(Constants.LogicalCheckEventsTitle.ItemUpdating);
                fldCheckWhen.DefaultValue = defaultChoices.ToString();

                // Update
                fldCheckWhen.Update();

                // Field successfully added
                return true;
            }
            catch { return false; }
        }

        public bool AddFieldValidate()
        {
            try
            {
                string newFieldName = Constants.LogicListFieldName.Validate;
                LogicList.Fields.Add(newFieldName, SPFieldType.Boolean, true);

                var fld = LogicList.Fields[newFieldName] as SPFieldBoolean;
                fld.DefaultValue = "1";
                fld.Description = "If checked this will validate this logic item. " +
                                    "For example it will validate if list exists or not. " +
                                    "This can be turned off if you want to create list later or " +
                                    "doing bulk import of logic";
                fld.Update();

                return true;
            }
            catch { return false; }
        }
    }
}
